@@ -42,10 +42,8 @@ import time
 from base64 import b64decode, b64encode
 from contextlib import contextmanager
 from json import load
-from threading import Thread
 
 import apt  # for independent apt.Cache
-import notifier.threads
 
 import univention.management.console as umc
 import univention.management.console.modules as umcm
@@ -59,14 +57,14 @@ from univention.appcenter.settings import FileSetting, PasswordFileSetting
 from univention.appcenter.ucr import ucr_instance, ucr_save
 from univention.appcenter.udm import _update_modules
 from univention.appcenter.utils import (
-    app_is_running, call_process, docker_bridge_network_conflict, docker_is_running, find_hosts_for_master_packages,
-    get_local_fqdn, resolve_dependencies, send_information,
+    app_is_running, call_process, docker_bridge_network_conflict, docker_is_running, get_local_fqdn,
+    resolve_dependencies, send_information,
 )
 from univention.lib.package_manager import LockError, PackageManager
-from univention.lib.umc import Client, ConnectionError, Forbidden, HTTPError
+from univention.lib.umc import Client, ConnectionError, HTTPError
 from univention.management.console.log import MODULE
 from univention.management.console.modules.decorators import (
-    multi_response, require_password, sanitize, sanitize_list, simple_response, threaded,
+    SimpleThread, multi_response, require_password, sanitize, sanitize_list, simple_response, threaded,
 )
 from univention.management.console.modules.mixins import ProgressMixin
 from univention.management.console.modules.sanitizers import (
@@ -219,7 +217,7 @@ class Instance(umcm.Base, ProgressMixin):
         info = get_action('info')
         return {'compatible': info.is_compatible(version, function=function), 'version': info.get_ucs_version()}
 
-    def _remote_appcenter(self, host, function=None):
+    def _remote_appcenter(self, request, host, function=None):
         if host is None:
             raise ValueError('Cannot connect to None')
         if not host.endswith('.%s' % self.ucr.get('domainname')):
@@ -229,7 +227,7 @@ class Instance(umcm.Base, ProgressMixin):
         if function is not None:
             opts['function'] = function
         try:
-            client = Client(host, self.username, self.password)
+            client = Client(host, request.username, request.password)
             response = client.umc_command('appcenter/version2', opts)
         except (HTTPError) as exc:
             raise umcm.UMC_Error(_('Problems connecting to {0} ({1}). Please update {0}!').format(host, exc.message))
@@ -276,8 +274,8 @@ class Instance(umcm.Base, ProgressMixin):
         settings=DictSanitizer({}, required=True),
         dry_run=BooleanSanitizer(),
     )
-    @simple_response(with_progress=True)
-    def run(self, progress, apps, auto_installed, action, hosts, settings, dry_run):
+    @simple_response(with_progress=True, with_request=True)
+    def run(self, request, progress, apps, auto_installed, action, hosts, settings, dry_run):
         localhost = get_local_fqdn()
         ret = {}
         if dry_run:
@@ -287,7 +285,7 @@ class Instance(umcm.Base, ProgressMixin):
                     ret[host] = self._run_local_dry_run(_apps, action, {}, progress)
                 else:
                     try:
-                        ret[host] = self._run_remote_dry_run(host, _apps, action, auto_installed, {}, progress)
+                        ret[host] = self._run_remote_dry_run(request, host, _apps, action, auto_installed, {}, progress)
                     except umcm.UMC_Error:
                         ret[host] = {'unreachable': [app.id for app in _apps]}
         else:
@@ -301,7 +299,7 @@ class Instance(umcm.Base, ProgressMixin):
                     if host == localhost:
                         host_result[app.id] = self._run_local(app, action, _settings, auto_installed, progress)
                     else:
-                        host_result[app.id] = self._run_remote(host, app, action, auto_installed, _settings, progress)[app.id]
+                        host_result[app.id] = self._run_remote(request, host, app, action, auto_installed, _settings, progress)[app.id]
                     if not host_result[app.id]['success']:
                         break
         return ret
@@ -355,18 +353,18 @@ class Instance(umcm.Base, ProgressMixin):
         finally:
             action.logger.removeHandler(handler)
 
-    def _run_remote_dry_run(self, host, apps, action, auto_installed, settings, progress):
-        return self._run_remote_logic(host, apps, action, auto_installed, settings, progress, dry_run=True)
+    def _run_remote_dry_run(self, request, host, apps, action, auto_installed, settings, progress):
+        return self._run_remote_logic(request, host, apps, action, auto_installed, settings, progress, dry_run=True)
 
-    def _run_remote(self, host, app, action, auto_installed, settings, progress):
-        return self._run_remote_logic(host, [app], action, auto_installed, settings, progress, dry_run=False)
+    def _run_remote(self, request, host, app, action, auto_installed, settings, progress):
+        return self._run_remote_logic(request, host, [app], action, auto_installed, settings, progress, dry_run=False)
 
-    def _run_remote_logic(self, host, apps, action, auto_installed, settings, progress, dry_run):
+    def _run_remote_logic(self, request, host, apps, action, auto_installed, settings, progress, dry_run):
         if len(apps) == 1:
             progress.title = _('%s: Connecting to %s') % (apps[0].name, host)
         else:
             progress.title = _('%d Apps: Connecting to %s') % (len(apps), host)
-        client = self._remote_appcenter(host, function='appcenter/run')
+        client = self._remote_appcenter(request, host, function='appcenter/run')
         opts = {'apps': [str(app) for app in apps], 'auto_installed': auto_installed, 'action': action, 'hosts': {host: [app.id for app in apps]}, 'settings': settings, 'dry_run': dry_run}
         progress_id = client.umc_command('appcenter/run', opts).result['id']
         while True:
@@ -450,10 +448,10 @@ class Instance(umcm.Base, ProgressMixin):
 
     @require_apps_update
     @require_password
-    @simple_response(with_progress=True)
-    def sync_ldap(self):
+    @simple_response(with_progress=True, with_request=True)
+    def sync_ldap(self, request):
         register = get_action('register')
-        register.call(username=self.username, password=self.password)
+        register.call(username=request.username, password=request.password)
 
     # used in updater-umc
     @simple_response
@@ -556,180 +554,6 @@ class Instance(umcm.Base, ProgressMixin):
         except LockError:
             raise umcm.UMC_Error(_('Another package operation is in progress'))
 
-    def _install_master_packages_on_hosts(self, app, function):
-        if function.startswith('upgrade'):
-            remote_function = 'update-schema'
-        else:
-            remote_function = 'install-schema'
-        master_packages = app.default_packages_master
-        if not master_packages:
-            return
-        hosts = find_hosts_for_master_packages()
-        all_hosts_count = len(hosts)
-        package_manager = get_package_manager()
-        package_manager.set_max_steps(all_hosts_count * 200)  # up to 50% if all hosts are installed
-        # maybe we already installed local packages (on master)
-        if self.ucr.get('server/role') == 'domaincontroller_master':
-            # TODO: set_max_steps should reset _start_steps. need function like set_start_steps()
-            package_manager.progress_state._start_steps = all_hosts_count * 100
-        for host, host_is_master in hosts:
-            package_manager.progress_state.info(_('Installing LDAP packages on %s') % host)
-            try:
-                if not self._install_master_packages_on_host(app, remote_function, host):
-                    error_message = 'Unable to install %r on %s. Check /var/log/univention/management-console-module-appcenter.log on the host and this server. All errata updates have been installed on %s?' % (master_packages, host, host)
-                    raise Exception(error_message)
-            except Exception as e:
-                MODULE.error('%s: %s' % (host, e))
-                if host_is_master:
-                    role = 'Primary Directory Node'
-                else:
-                    role = 'Backup Directory Node'
-                # ATTENTION: This message is not localised. It is parsed by the frontend to markup this message! If you change this message, be sure to do the same in AppCenterPage.js
-                package_manager.progress_state.error('Installing extension of LDAP schema for %s seems to have failed on %s %s' % (app.component_id, role, host))
-                if host_is_master:
-                    raise  # only if host_is_master!
-            finally:
-                package_manager.add_hundred_percent()
-
-    def _install_master_packages_on_host(self, app, function, host):
-        client = Client(host, self.username, self.password)
-        result = client.umc_command('appcenter/invoke', {'function': function, 'application': app.id, 'force': True, 'dont_remote_install': True}).result
-        if result['can_continue']:
-            all_errors = self._query_remote_progress(client)
-            return len(all_errors) == 0
-        else:
-            MODULE.warn('%r' % result)
-            return False
-
-    def _install_dry_run_remote(self, app, function, dont_remote_install, force):
-        MODULE.process('Invoke install_dry_run_remote')
-        self.ucr.load()
-        if function.startswith('upgrade'):
-            remote_function = 'update-schema'
-        else:
-            remote_function = 'install-schema'
-
-        master_packages = app.default_packages_master
-
-        # connect to Primary/Backup Nodes
-        unreachable = []
-        hosts_info = {}
-        remote_info = {
-            'master_unreachable': False,
-            'problems_with_hosts': False,
-            'serious_problems_with_hosts': False,
-        }
-        dry_run_threads = []
-        info = get_action('info')
-        if master_packages and not dont_remote_install:
-            hosts = find_hosts_for_master_packages()
-            # checking remote host is I/O heavy, so use threads
-            #   "global" variables: unreachable, hosts_info, remote_info
-
-            def _check_remote_host(app_id, host, host_is_master, username, password, force, remote_function):
-                MODULE.process('Starting dry_run for %s on %s' % (app_id, host))
-                MODULE.process('%s: Connecting...' % host)
-                try:
-                    client = Client(host, username, password)
-                except (ConnectionError, HTTPError) as exc:
-                    MODULE.warn('_check_remote_host: %s: %s' % (host, exc))
-                    unreachable.append(host)
-                    if host_is_master:
-                        remote_info['master_unreachable'] = True
-                else:
-                    MODULE.process('%s: ... done' % host)
-                    host_info = {}
-                    MODULE.process('%s: Getting version...' % host)
-                    try:
-                        host_version = client.umc_command('appcenter/version', {'version': info.get_compatibility()}).result
-                    except Forbidden:
-                        # command is not yet known (older app center)
-                        MODULE.process('%s: ... forbidden!' % host)
-                        host_version = None
-                    except (ConnectionError, HTTPError) as exc:
-                        MODULE.warn('%s: Could not get appcenter/version: %s' % (host, exc))
-                        raise
-                    except Exception as exc:
-                        MODULE.error('%s: Exception: %s' % (host, exc))
-                        raise
-                    MODULE.process('%s: ... done' % host)
-                    host_info['compatible_version'] = info.is_compatible(host_version)
-                    MODULE.process('%s: Invoking %s ...' % (host, remote_function))
-                    try:
-                        host_info['result'] = client.umc_command('appcenter/invoke_dry_run', {
-                            'function': remote_function,
-                            'application': app_id,
-                            'force': force,
-                            'dont_remote_install': True,
-                        }).result
-                    except Forbidden:
-                        # command is not yet known (older app center)
-                        MODULE.process('%s: ... forbidden!' % host)
-                        host_info['result'] = {'can_continue': False, 'serious_problems': False}
-                    except (ConnectionError, HTTPError) as exc:
-                        MODULE.warn('Could not get appcenter/version: %s' % (exc,))
-                        raise
-                    MODULE.process('%s: ... done' % host)
-                    if not host_info['compatible_version'] or not host_info['result']['can_continue']:
-                        remote_info['problems_with_hosts'] = True
-                        if host_info['result']['serious_problems'] or not host_info['compatible_version']:
-                            remote_info['serious_problems_with_hosts'] = True
-                    hosts_info[host] = host_info
-                MODULE.process('Finished dry_run for %s on %s' % (app_id, host))
-
-            for host, host_is_master in hosts:
-                thread = Thread(target=_check_remote_host, args=(app.id, host, host_is_master, self.username, self.password, force, remote_function))
-                thread.start()
-                dry_run_threads.append(thread)
-
-        result = {}
-
-        for thread in dry_run_threads:
-            thread.join()
-        MODULE.process('All %d threads finished' % (len(dry_run_threads)))
-
-        result['unreachable'] = unreachable
-        result['hosts_info'] = hosts_info
-        result.update(remote_info)
-        return result
-
-    def _query_remote_progress(self, client):
-        all_errors = set()
-        number_failures = 0
-        number_failures_max = 20
-        host = client.hostname
-        while True:
-            try:
-                result = client.umc_command('appcenter/progress').result
-            except (ConnectionError, HTTPError) as exc:
-                MODULE.warn('%s: appcenter/progress returned an error: %s' % (host, exc))
-                number_failures += 1
-                if number_failures >= number_failures_max:
-                    MODULE.error('%s: Remote App Center cannot be contacted for more than %d seconds. Maybe just a long Apache Restart? Presume failure! Check logs on remote machine, maybe installation was successful.' % number_failures_max)
-                    return False
-                time.sleep(1)
-                continue
-            else:
-                # everything okay. reset "timeout"
-                number_failures = 0
-            MODULE.info('Result from %s: %r' % (host, result))
-            info = result['info']
-            steps = result['steps']
-            errors = ['%s: %s' % (host, error) for error in result['errors']]
-            if info:
-                self.package_manager.progress_state.info(info)
-            if steps:
-                steps = float(steps)  # bug in package_manager in 3.1-0: int will result in 0 because of division and steps < max_steps
-                self.package_manager.progress_state.percentage(steps)
-            for error in errors:
-                if error not in all_errors:
-                    self.package_manager.progress_state.error(error)
-                    all_errors.add(error)
-            if result['finished'] is True:
-                break
-            time.sleep(0.1)
-        return all_errors
-
     @threaded
     def keep_alive(self, request):
         """
@@ -861,8 +685,8 @@ class Instance(umcm.Base, ProgressMixin):
                     def _finished(thread, result):
                         if isinstance(result, BaseException):
                             MODULE.warn('Exception during %s %s: %r' % (function, packages, str(result)))
-                    thread = notifier.threads.Simple('invoke', notifier.Callback(_thread, self.package_manager, function, packages), _finished)
-                    thread.run()
+                    thread = SimpleThread('invoke', _thread, _finished)
+                    thread.run(self.package_manager, function, packages)
                 else:
                     self.package_manager.set_finished()  # nothing to do, ready to take new commands
         except LockError:
